package cd.casic.ci.process.engine.manager.impl;

import cd.casic.ci.process.engine.manager.PostHandlerManager;
import cd.casic.ci.process.engine.postHandler.PipExecutePostHandler;
import cd.casic.ci.process.process.dataObject.history.PipPipelineHisInstance;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

@Component
public class MemoryPostHandlerManager implements PostHandlerManager {
    private ConcurrentHashMap<String, List<PipExecutePostHandler>> handlerMap = new ConcurrentHashMap<>();
    @Override
    public void registerPostHandler(PipExecutePostHandler handler) {
        if (handler==null|| StringUtils.isNotEmpty(handler.getPipelineId())) {
            throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"注册后置处理器失败");
        }
        List<PipExecutePostHandler> orDefault = handlerMap.getOrDefault(handler.getPipelineId(), new CopyOnWriteArrayList<>());
        orDefault.add(handler);
    }

    @Override
    public void executePostHandler(PipPipelineHisInstance hisInstance) {
        String pipelineId = hisInstance.getPipelineId();
        List<PipExecutePostHandler> pipExecutePostHandlers = handlerMap.get(pipelineId);
        if (CollectionUtils.isEmpty(pipExecutePostHandlers)) {
            return;
        }
        for (PipExecutePostHandler postHandler : pipExecutePostHandlers) {
            postHandler.executeAfterDone(hisInstance);
        }
    }
}
